@@ -132,7 +132,8 @@ class CustomLossHandler:
 class MetricHandlerData:
     name: str
     arg_names: List[str]
-    direction: Optional[MetricDirection] = MetricDirection.Downward
+    direction: Union[None, MetricDirection, Dict[str, MetricDirection]] = MetricDirection.Downward
+    compute_insights: Union[bool, Dict[str, bool]] = True
 
 
 @dataclass
@@ -171,6 +172,7 @@ class InputHandler(DatasetBaseHandler):
     shape: Optional[List[int]] = None
     channel_dim: Optional[int] = -1
 
+
 @dataclass
 class GroundTruthHandler(DatasetBaseHandler):
     shape: Optional[List[int]] = None
